//
//  main.c
//  CryptTool
//
//  Created by Alexandre Colucci on 22/07/16.
//  Copyright © 2016 Alexandre Colucci. All rights reserved.
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// Include for mach-o structures
#include <mach-o/loader.h>

// Our encryption functions
#include "encryption.h"


static void PrintUsage()
{
	fprintf(stderr, "Usage: CryptTool [-decrypt|-encrypt] [binary path]\n");
	fprintf(stderr, "If -decrypt and -encrypt are not specified, -decrypt is used\n");
}


int main(int argc, const char * argv[])
{
#if !__LP64__
	#error This tool can only run as 64-bit intel
	exit(1);
#endif

	if(argc < 2)
	{
		PrintUsage();
		exit(1);
	}
	
	// Parse the arguments
	
	CCOperation operation = kCCEncrypt;
	const char *binaryPath = NULL;
	
	for (int i = 1; i < argc; i++)
	{
		if (!strcmp(argv[i], "-decrypt"))
		{
			operation = kCCDecrypt;
		}
		else if (!strcmp(argv[i], "-encrypt"))
		{
			operation = kCCEncrypt;
		}
		else
		{
			binaryPath = argv[i];
			break;
		}
	}
	
	// Open the source file
	FILE *srcFile = fopen(binaryPath, "r");
	if (srcFile == NULL)
	{
		fprintf(stderr, "Error: Could not open binary %s\n", binaryPath);
		exit(1);
	}
	
	// Read the source file and store it into a buffer
	fseek(srcFile, 0, SEEK_END);
	long fileLen = ftell(srcFile);
	fseek(srcFile, 0, SEEK_SET);
	
	uint8_t *fileBuffer = (uint8_t *) calloc(fileLen, 1);
	if (fileBuffer == NULL)
	{
		fprintf(stderr, "Error: Could not allocate buffer\n");
		exit(1);
	}
	
	if (fread(fileBuffer, 1, fileLen, srcFile) != fileLen)
	{
		fprintf(stderr, "Error: Could not read binary\n");
		
		// Cleanup
		free(fileBuffer);
		fileBuffer = NULL;
		
		fclose(srcFile);
		srcFile = NULL;
		
		exit(1);
	}
	
	fclose(srcFile);
	srcFile = NULL;
	
	
	// Get the mach header
	struct mach_header_64 *machHeader = (struct mach_header_64 *) fileBuffer;
	
	// Encrypt/Decrypt the mach-o file
	if (machHeader->magic == MH_MAGIC_64)
	{
		EncryptDecryptMachoFile(machHeader, operation);
	}
	else
	{
		fprintf(stderr, "Error: Only 64-bit mach-o file are supported\n");
		
		// Cleanup
		free(fileBuffer);
		fileBuffer = NULL;
		
		exit(1);
	}
	
	// Write the encrypted/decrypted buffer to disk
	FILE *dstFile = fopen(binaryPath, "w");
	if (dstFile == NULL)
	{
		fprintf(stderr, "Error: Unable to open destination file %s\n", binaryPath);
		
		// Cleanup
		free(fileBuffer);
		fileBuffer = NULL;
		
		exit(1);
	}
	
	if (fwrite(fileBuffer, 1, fileLen, dstFile) != fileLen)
	{
		fprintf(stderr, "Error: Unable to write destination file\n");
		
		// Cleanup
		free(fileBuffer);
		fileBuffer = NULL;
		
		fclose(dstFile);
		dstFile = NULL;
		
		exit(1);
	}
	
	fclose(dstFile);
	dstFile = NULL;
	
	free(fileBuffer);
	fileBuffer = NULL;
	
    return 0;
}
