//
//  main.c
//  CryptedHelloWorld
//
//  Created by Alexandre Colucci on 22/07/16.
//  Copyright © 2016 Alexandre Colucci. All rights reserved.
//

#include <stdio.h>
#include <mach-o/dyld.h>
#include "encryption.h"

//
// Constructor function called by dyld before running main().
// This function takes care of decrypting the (__TEXT, __text) section before main() is called.
//
// Note 2 important points:
//
// 1- This is a constructor function and as thus you can see the attribute ((constructor))
//
// 2- This function can't be stored in the (__TEXT, __text) section otherwise it would be encrypted.
//		It is explicitely stored in a custom (__TEXT, __timac) section which we don't encrypt.
//		This is done through the attribute ((section("__TEXT,__timac"))).
//
void __attribute__((constructor)) __attribute__((section("__TEXT,__timac"))) decryptTextSection()
{
	printf("\n\n");
	fprintf(stderr, "*** Constructor called to decrypt sections\n");
	
	// Decrypt the main mach-o file
	struct mach_header_64 *header = (struct mach_header_64 *)_dyld_get_image_header(0);
	EncryptDecryptMachoFile(header, kCCDecrypt);
	
	fprintf(stderr, "*** The sections should now be decrypted. main() will be called soon.\n");
}

//
// This is the actual program. It just prints some strings.
//
int main(int argc, const char * argv[])
{
#if !__LP64__
	#error This proof on concept only supports 64-bit intel
#endif

	printf("\n\n");
	printf("------------------\n");
	printf("Hello, World!\n");
	printf("------------------\n");
	printf("\n\n");
	
    return 0;
}
